package SketchAggSensor;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import java.util.*;

public class Clock {

    Vector listeners;
    ClockListener finalListener;

    int time = 0;

    /** Constructor */
    public Clock() {
        listeners = new Vector();
        finalListener = null;
    }

    /** Register the specified listener with the clock */
    public void addListener(ClockListener l) {
        listeners.addElement(l);
    }

    /** Allow a specific listener to be guaranteed that it is
        the last to be updated
    */
    public void setFinalListener(ClockListener l) {
        finalListener = l;
    }

    /** Advance the clock, notifying all of the listeners (in unspecified order),
        with the finalListener notified last.
    */
    public void nextTime() {
        Enumeration e = listeners.elements();
        time++;
        while (e.hasMoreElements()) {
            ClockListener l = (ClockListener)e.nextElement();
            l.setTime(time);
        }
        if (finalListener != null)
        finalListener.setTime(time);
    }

    /** Return the current time on the clock */
    public int getTime() {
        return time;
    }

    /** Return the number of epochs represented by the current time.
        Currently, epochs are two time intervals...<p>
        Why do this?  <p>
        So that sensors can all be notified of a new epoch, send out
        and process messages before any other sensors have moved on to the
        next epoch.
    */
    public static int getEpoch(int time) {
        return time >> 1;
    }
}