package SketchAggSensor.agg;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */
import SketchAggSensor.*;
import SketchAggSensor.sensor.*;
import java.util.*;

public interface Aggregator {
  static final int INT_BYTES = 4;
    static final int CHAR_BYTES = 1;
    static final int BOOL_BYTES = 1;
    static final int FLOAT_BYTES = 8;

    /** Combine the specified sensor reading and the aggregation object
        @param sensor The sensor whose reading is to be added to the aggregate object
        @param agg The aggregation object (may be modified and returned)
        @return The new aggregate object (may be a reference to agg)
        @throws InvalidAggregateException if agg is not the appropriate aggregate type
    */
    public Object combine(Sensor sensor, Object agg) throws InvalidAggregateException;

    /** Combine two aggregates
        @param agg1 The first aggregate object (may be modified and returned)
        @param agg2 The second aggregate object
        @return The new aggregate object (may be a reference to agg1)
        @throws InvalidAggregateException if agg1 or agg2 is not an appropriate aggregate type
    */
    public Object combine(Object agg1, Object agg2) throws InvalidAggregateException;

    /** @return an aggregate object with no entries
        @param s The sensor that produced the aggregate value
    */
    public Object emptyAgg(Sensor s);

    /** @return the integer aggregate value of an aggregate object */
    public int getValue(Object agg) throws InvalidAggregateException;

    /** @return the non-integer aggregate value of an aggregate object */
    public Object getValueObj(Object agg) throws InvalidAggregateException;

    /** @return the number of sensor contributing to this aggregate,
        or -1 if unknown */
    public int getCount(Object agg) throws InvalidAggregateException;

    /** @return the number of bytes this aggregate represents */
    public int numBytes(Object agg) throws InvalidAggregateException;

    /** Split this aggregate up into divisor parts, so that it can be sent to
       multiple parents.  Modifies agg.  May not be supported by all aggregates.
       @param agg The aggregate to divide -- modified!
       @throws InvalidAggregateException if unsupported, or if agg is not an appropriate
               aggregate type.

    */
    public void divide(Object agg, int divisor) throws InvalidAggregateException;

    /** @return a copy of agg */
    public Object cloneAgg(Object agg) throws InvalidAggregateException;

    /** @return A "hypothesis" aggregate value that will be used to suppress the
        aggregate values of sensors above or below this threshold.  A hack, since
        the aggregation function doesn't have access to any other aggregation values
        when it makes this decision.
    */
    public Object makeGuess();

}
