package SketchAggSensor.agg;

import SketchAggSensor.sensor.Sensor;
import SketchAggSensor.InvalidAggregateException;
import java.util.*;
/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

public class CountListAgg implements Aggregator {

  public Object combine(Sensor sensor, Object agg) throws InvalidAggregateException {
    int reading = sensor.getReading(sensor.QUERY_FIELD);
    if (!(agg instanceof CountListAggObj)) throw new InvalidAggregateException();
    Integer sensorID = new Integer(sensor.getId());
    ((CountListAggObj)agg).cntMap.put(sensorID,sensor);
    return agg;
  }

  public Object combine(Object agg1, Object agg2) throws InvalidAggregateException {
    if (!(agg1 instanceof CountListAggObj)) throw new InvalidAggregateException();
    if (!(agg2 instanceof CountListAggObj)) throw new InvalidAggregateException();
    CountListAggObj c1 = (CountListAggObj)agg1;
    CountListAggObj c2 = (CountListAggObj)agg2;
    CountListAggObj newAgg = new CountListAggObj();
    newAgg.cntMap = c1.cntMap;
    Iterator keySetC2 = c2.cntMap.keySet().iterator();
    while(keySetC2.hasNext())
    {
      Integer idNext = (Integer)keySetC2.next();
      newAgg.cntMap.put(idNext,c2.cntMap.get(idNext));
    }
    return newAgg;
  }
  public Object emptyAgg(Sensor s) {
    return new CountListAggObj();
  }
  public int getValue(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof CountListAggObj)) throw new InvalidAggregateException();
    return ((CountListAggObj)agg).cntMap.size();
  }

  public Object getValueObj(Object agg) throws InvalidAggregateException {
    return null;
  }
  public int getCount(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof CountListAggObj)) throw new InvalidAggregateException();
    return ((CountListAggObj)agg).cntMap.size();
  }
  public int numBytes(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof CountListAggObj)) throw new InvalidAggregateException();
    CountListAggObj c = (CountListAggObj)agg;
    return c.cntMap.size()*2*2;
  }
  public void divide(Object agg, int divisor) throws InvalidAggregateException {

  }
  public Object cloneAgg(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof CountListAggObj)) throw new InvalidAggregateException();
    CountListAggObj newAgg = new CountListAggObj();
    newAgg.cntMap = ((CountListAggObj)agg).cntMap;
    return newAgg;
  }
  public Object makeGuess() {
    return new CountListAggObj();
  }

  class CountListAggObj {
      public HashMap cntMap = new HashMap();
      public String toString() {
          String s = "" + cntMap.size();
          return s;
     }
   }
}