package SketchAggSensor.agg;

import SketchAggSensor.sensor.Sensor;
import SketchAggSensor.InvalidAggregateException;
import SketchAggSensor.sketch.*;
import java.util.Random;
/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

public class SumSketchAgg implements Aggregator {

  public Random random;

  public Object combine(Sensor sensor, Object agg) throws InvalidAggregateException {
       int reading = sensor.getReading(sensor.QUERY_FIELD);
       if (!(agg instanceof SumSketch)) throw new InvalidAggregateException();
       {
         random = new Random(sensor.queryID);
         int key =0;
         for (int i=0;i<sensor.getId();i++)
           key = random.nextInt();
         ((SumSketch)agg).insert(key,reading);
         //((SumSketch)agg).insert(sensor.sketchInsertKey,reading);
       }
       return agg;
  }
  public Object combine(Object agg1, Object agg2) throws InvalidAggregateException {
    if (!(agg1 instanceof SumSketch)) throw new InvalidAggregateException();
    if (!(agg2  instanceof SumSketch)) throw new InvalidAggregateException();

    SumSketch s1 = (SumSketch) agg1;
    SumSketch s2 = (SumSketch) agg2;
    SumSketch newAgg = new SumSketch(Sketch.mergeSketch(s1,s2));
    return newAgg;
  }

  public Object emptyAgg(Sensor s) {
    SumSketch agg = new SumSketch(Sensor.numBitMaps,Sensor.bitMapLength);
    return agg;
  }

  public int getValue(Object agg) throws InvalidAggregateException {
    if (! (agg instanceof SumSketch)) throw new InvalidAggregateException();
    return (int) ((SumSketch)agg).estimate();
  }

  public Object getValueObj(Object agg) throws InvalidAggregateException {
    return null;
  }
  public int getCount(Object agg) throws InvalidAggregateException {
    return 1;
  }
  public int numBytes(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof SumSketch)) throw new InvalidAggregateException();
    SumSketch s = (SumSketch) agg;
    int totalBits =0;
    if (Sensor.COMPRESSION_GROUP == Sensor.numBitMaps)
    {
      if (Sensor.COMPRESSION_SCHEME == 1) //our compression algo
      {  s.SetKey();
         totalBits = (s.bitMapLength-s.key0-s.key1)*s.numBitMap;
         totalBits +=4;
      }
      else if (Sensor.COMPRESSION_SCHEME == 2) //ANF compression algo
      {
        s.compressionANF(0,s.numBitMap);
        totalBits = s.bitMapLength*s.numBitMap-s.key0-s.key1+4;
      }
    }
    else
    {
      int numGroup = Sensor.numBitMaps / Sensor.COMPRESSION_GROUP;
      for (int k= 0; k<numGroup; k++)
      {
        int start = Sensor.COMPRESSION_GROUP*k;
        int stop = Sensor.COMPRESSION_GROUP*(k+1);
        if (Sensor.COMPRESSION_SCHEME == 1)
        {  s.SetKey(start,stop);
           totalBits += (s.bitMapLength-s.key0-s.key1)*Sensor.COMPRESSION_GROUP;
           totalBits +=4;
        }
        else if (Sensor.COMPRESSION_SCHEME ==2 )
        {
          s.compressionANF(start,stop);
          totalBits += s.bitMapLength*Sensor.COMPRESSION_GROUP-s.key0-s.key1;
          totalBits += 4;
        }
      }
    }
    return totalBits;
  }

  public void divide(Object agg, int divisor) throws InvalidAggregateException {

  }
  public Object cloneAgg(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof SumSketch)) throw new InvalidAggregateException();
    SumSketch s = (SumSketch)agg;
    SumSketch newAgg = (SumSketch)Sketch.cloneSketch(s);
    return newAgg;
  }
  public Object makeGuess() {
    SumSketch guess = new SumSketch(2,32);
    return guess;
  }
}
