package SketchAggSensor.loglogsketch;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import java.util.HashMap;
import java.util.Iterator;

public class Sketch
  {
    final static double varphi = 0.77351;
    public int numBitMap;
    public int k;
    public int bitMapLength = 24;
    protected int bitmaps[];
    public int counterM[];
    public boolean  sketches[][];
    public double eulerconstant =0.577215665;
    public double ar=0.0;
    public double am=0.0;
    public double dm=0.0;
    public int key0;
    public int key1;

    public Sketch()
    {
      this(1);
    }

    public Sketch(int kIn)
    {
      if(kIn < 1)
      {
          throw new IllegalArgumentException("m must be a positive integer");
      }
      this.k = kIn;
      this.numBitMap = (int)Math.pow(2,(double)kIn);
      this.counterM = new int[this.numBitMap];
      for (int i=0;i<this.numBitMap;i++)
        counterM[i]=0;
      bitmaps = new int[bitMapLength];
        for (int j=0;j<bitMapLength;j++)
          bitmaps[j]=0;
      ar = Math.pow(Math.E, -1.0*eulerconstant)*Math.sqrt(2.0);
      dm = (double)this.numBitMap;
      //double ar = 0.79402;
      double b =  2.0*Math.pow(Math.PI,2.0)+Math.pow(Math.log(2.0),2.0);
      am = ar - b/(24.0*dm);
      sketches = new boolean[this.numBitMap][4];
      //System.out.println(ar+" "+ am);
    }

    // based on Flajolet's loglog pseudo-code
    public synchronized double estimate()
    {
       double total = 0;
       for (int i=0;i<this.numBitMap;i++)
         total=total+(double)counterM[i];
       double avg = total/dm;
       double p = Math.pow(2.0,avg);
       return am*dm*p;
    }

    public double entropy()
    {
       HashMap map = new HashMap();
       double entropy =0.0;
       for (int i=0;i<this.numBitMap;i++)
       {
         Integer cm = new Integer(counterM[i]);
         Object objtmp = map.get(cm);
         if (objtmp==null)
           map.put(cm,new Integer(1));
         else
         {
           Integer tmpInt = (Integer)objtmp;
           map.put(cm,new Integer(tmpInt.intValue()+1));
         }
       }
       Iterator it = map.keySet().iterator();
       while (it.hasNext())
       {
         Integer key = (Integer)it.next();
         Integer count = (Integer)map.get(key);
         double p = count.doubleValue()/(double)numBitMap;
         entropy+=p*Math.log(p);
       }
       return -1.0*entropy;
    }

    public double compression()
    {
      for (int i=0;i<this.numBitMap;i++)
        printBinary(i);
      return 1.0;
    }

    public void printSketch ()
    {
      for (int i=0; i < this.numBitMap;i++)
      {
          System.out.println(this.counterM[i]+"\n");
      }
    }

    private void printBinary(int i)
    {
      for (int sh = 3; sh >= 0; sh--) {
        int bit = (counterM[i] >> sh) & 1;
        System.out.print( bit);
        if (bit==1)
          sketches[i][sh]=true;
        else
          sketches[i][sh]=false;
      }
      System.out.println("   "+counterM[i]);
    }

    public void compressionANF(int start, int stop)
    {
      key1 = 0;
      key0 = 0;
      boolean tmpkeyarr[];
      int range = stop - start;
      int length = this.bitMapLength * range;
      tmpkeyarr = new boolean[length];
      int k = 0;
      for (int j = 0; j < this.bitMapLength; j++) {
        for (int i = start; i < stop; i++) {
          tmpkeyarr[k++] = this.sketches[i][j];
        }
      }
      k = 0;
      while (tmpkeyarr[k] == true) {
        k++;
        if (k == length)
          break;
      }
      key1 = k;
      k = length - 1;
      while (tmpkeyarr[k] == false) {
        k--;
        if (k < 0)
          break;
      }
      key0 = length - 1 - k;
    }

    public static Sketch mergeSketch(Sketch sketA, Sketch sketB)
    {
      Sketch outputSketch = new Sketch(sketA.k);
      for (int i=0; i < sketA.numBitMap;i++)
      {
        if (sketA.counterM[i] > sketB.counterM[i])
               outputSketch.counterM[i] = sketA.counterM[i];
        else
               outputSketch.counterM[i] = sketB.counterM[i];
      }
      return outputSketch;
    }

    public static Sketch cloneSketch( Sketch sketA)
    {
      Sketch outputSketch = new Sketch(sketA.k);
      for (int i=0; i < sketA.numBitMap;i++)
          outputSketch.counterM[i]=sketA.counterM[i];
      return outputSketch;
    }
}
