package SketchAggSensor.message;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import SketchAggSensor.*;
import SketchAggSensor.sensor.*;
import java.util.*;

public class Message {

    /** The sender of the message */
     protected Sensor sender;

     /** The recipients of this message */
     protected Vector recipients;

     /** Is this a broadcast message?  Broadcase messages
         can be heard by all neighbors within hearing range --
         @see Topology
         @see RealisticTopology
     */
     protected boolean isBroadcast;

     /** Epoch this message was sent */
     protected int epoch;

     /** Number of bytes used in this message */
     protected int bytes;

    public boolean spanningTreeMsg;
    public int queryID;
    public int sketchInsertKey;

     /** Create a new message
         @param sender The sender of the message
         @param epoch The epoch this message was sent
         @param bytes The number of bytes of the message
     */
     public Message(Sensor sender, int epoch, int bytes, boolean broadcast) {
         isBroadcast = broadcast;
         recipients = new Vector();
         this.sender = sender;
         this.epoch = epoch;
         this.bytes = bytes;
         spanningTreeMsg = false;
     }

     /** Return the sender of the message */
     public Sensor getSender() {
         return sender;
     }

     /** Add the specified sensor id {@link Sensor#getId() Sensor.getId()}
         as a receipient of the message
     */
     public void addRecipient(int recID) {
         recipients.addElement(new Integer(recID));
     }

     /** @return Is this message a broadcast message ? */
     public boolean isBroadcast() {
         return isBroadcast;
     }

     /** @param val Set the broadcast flag to this value */
     public void setBroadcast(boolean val) {
         isBroadcast = val;
     }

     /** @param id The id of the sensor to look for
         @return true if the specified sensor id is a recipient
     */
     public boolean isRecipient(int id) {
         for (int i = 0; i < recipients.size(); i++) {
             if (((Integer)recipients.elementAt(i)).intValue() == id)
                 return true;
         }
         return false;
     }

     /** @return The number of bytes required to encode this message */
     public int numBytes() {
         return bytes;
     }

     /** @return The epoch in which this message was encoded */
     public int getEpoch() {
         return epoch;
     }

     public Vector getRecipients()
     {
       return this.recipients;
     }

     public boolean isSpanningTreeMsg()
     {
       return this.spanningTreeMsg;
     }

}