package SketchAggSensor.sensor;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */
import SketchAggSensor.*;
import SketchAggSensor.agg.*;
import SketchAggSensor.message.*;
import SketchAggSensor.topology.*;
import java.util.*;

public class NormalAggSensor extends Sensor implements ClockListener {

  Object thisAgg;
  Object thisAgg1;
  Aggregator agg;
  Aggregator agg1;
  int endEpoch;

  static final int LEVEL_UNKNOWN = -1;


   /** Constructor
       @param t The topology that the sensor uses to propagate messages
       @param agg The aggregation function to apply
       @param dist The sensor distribution to use (See {@link Sensor Sensor})
       @param parentReading unused
   */
   public NormalAggSensor(Topology t, int id, Aggregator agg, int dist) {
       super(t,id, dist);

       parents = new Vector();
       level = LEVEL_UNKNOWN;
       thisAgg = agg.emptyAgg(this);
       this.agg = agg;
   }

   public NormalAggSensor(Topology t, int id, Aggregator agg, Aggregator agg1, int dist) {
    super(t,id, dist);

    parents = new Vector();
    level = LEVEL_UNKNOWN;
    thisAgg = agg.emptyAgg(this);
    this.agg = agg;
    thisAgg1 = agg1.emptyAgg(this);
    this.agg1 = agg1;
    this.twoAggregator=true;
   }


   /** Set the time...
       Check to see if this is our appointed time to send our aggregate
       value -- if so, send it out.
   */
   public void setTime(int time) {
       int lastEpoch = Clock.getEpoch(this.time);

       super.setTime(time);
       if (lastEpoch < Clock.getEpoch(time)) {
           if (level != LEVEL_UNKNOWN && endEpoch == Clock.getEpoch(time)) {
               //report last aggregate
               //dont ferget to aggregate ourselves!
               try {
                   Object aggr = agg.combine(this, thisAgg);
                   Object aggr1 = null;
                   if (this.twoAggregator==true)
                     aggr1 = agg1.combine(this,thisAgg1);
                   if (Sensor.USING_DIVISOR)
                   {  agg.divide(aggr,this.parents.size());
                      if (this.twoAggregator==true)
                        agg1.divide(aggr1,this.parents.size());
                   }
                   AggMessage m;
                   if (this.twoAggregator==false)
                    m = new AggMessage(this, Clock.getEpoch(time) - 1, aggr , level, parents, agg.numBytes(aggr));
                   else
                    m = new AggMessage(this, Clock.getEpoch(time) - 1, aggr ,aggr1, level, parents, agg.numBytes(aggr));
                   Enumeration parentPoll = this.parents.elements();
                   while (parentPoll.hasMoreElements())
                   {
                     Integer next = (Integer)parentPoll.nextElement();
                     if (Sensor.r.nextDouble()>Sensor.LINK_FAILURE)
                       m.addRecipient(next.intValue());
                     //System.out.println("Sensor "+ this.getId()+" at ("+this.x+","+this.y+") Receients/Parents size:"+this.parents.size());
                   }
                   //System.out.println("Sensor "+ this.getId()+" at ("+this.x+","+this.y+") Receients size:"+ m.getRecipients().size());
                   if (Sensor.r.nextDouble()>Sensor.NODE_FAILURE)
                   {  m.setBroadcast(false);
                     sendMessage(m);
                   }
               } catch (InvalidAggregateException e) {
                   //dont expect this
               }
           }
       }
   }

   /** Called by {@link Topology Topology} when a message arrives for this sensor.
       Check to see if: <br>
       1) This is a request to aggregate -- if so, start aggregation
       2) This is a message from a child -- if so, add their aggregate value to our
          partial aggregate.
   */
   public void receiveMessage(Message m) {
       super.receiveMessage(m);

       if (m instanceof AggMessage) {
           AggMessage am = (AggMessage)m;

           if (level == LEVEL_UNKNOWN) {
               Object emptyAgg = agg.emptyAgg(this);
               //Object emptyAgg1 = agg1.emptyAgg(this);
               AggMessage newm = null;
               this.setQueryID(am.queryID);
               /*int k = this.getId()-am.getSender().getId();
               int key=0;
               Random rkey = new Random(am.sketchInsertKey);
               for (int j=0;j<k;j++)
                 key = rkey.nextInt();
               this.sketchInsertKey = key;*/
               //generate the sketchInsertkey which is the IDth number
               //in the random number sequence with initial key as the queryID
               //of this round
               level = am.getLevel() + 1;
               if (parents == null)
                 parents = new Vector();
               if (!this.isParent(am.getSender().getId()))
                   this.addParents(am.getSender().getId());
               //send result MAX_EPOCHS-level from now, so deeper nodes
               //report before their parents
               endEpoch = Clock.getEpoch(time) + (MAX_EPOCHS - level);
               //make sure we notify children

               try {
                   newm = new AggMessage(this, Clock.getEpoch(time) - 1, emptyAgg, level, parents, agg.numBytes(emptyAgg));
                   newm.spanningTreeMsg = true;
                   newm.queryID = am.queryID;
                   //newm.sketchInsertKey = this.sketchInsertKey;
                   //pass the sketchinsertKey
               } catch (InvalidAggregateException e) {
                   //shouln't happen
               }
               newm.setBroadcast(false); //parent should ignore!
               sendMessage(newm);
           }
           //multiple parents here!
           if (this.isMultipleParents())
           {  if (m.isSpanningTreeMsg())
              {
                if (m.getSender().getLevel() == level-1)
                {
                  if (!this.isParent(am.getSender().getId()))
                    this.addParents(m.getSender().getId());
                }
              }
           }
           if (!m.isBroadcast() && !m.isRecipient(id)) return;
           //System.out.println("Sensor " + id + " accepts " + m + ", my level is : " + level);
           if (am.getLevel() == level + 1) {
               try {
                   numMsgs++;
                   thisAgg = agg.combine(thisAgg, am.getAggregate());
                   if (this.twoAggregator ==true)
                     thisAgg1 = agg1.combine(thisAgg1,am.getAggregate1());
               } catch (InvalidAggregateException e) {
                   //dont expect this
               }
           }
       }
   }

   public int getAgg() {

       try {
           return agg.getValue(thisAgg);
       } catch (InvalidAggregateException e) {
           System.err.println("Error: " + e);
           e.printStackTrace();
       }
       return 0;
   }

   public int getAgg1() {

    try {
        return agg1.getValue(thisAgg1);
    } catch (InvalidAggregateException e) {
        System.err.println("Error: " + e);
        e.printStackTrace();
    }
    return 0;
}


   /** Return the non-integer aggregate value from the last epoch */
   public Object getAggObj() {
       try {
           return agg.getValueObj(thisAgg);
       } catch (InvalidAggregateException e) {
           System.err.println("Error: " + e);
           e.printStackTrace();
       }
       return null;
   }

   public Object getAggObj1() {
    try {
        return agg1.getValueObj(thisAgg1);
    } catch (InvalidAggregateException e) {
        System.err.println("Error: " + e);
        e.printStackTrace();
    }
    return null;
}


}