package SketchAggSensor.sketch;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import java.util.Random;

public class CountSketch extends Sketch
{
  public CountSketch()
  {
    super();
  }

  public CountSketch(int mIn)
  {
    super(mIn);
  }

  public CountSketch (int mIn, int length)
  {
    super(mIn,length);
  }

  public CountSketch (Sketch sketch)
  {
    super(sketch.numBitMap,sketch.bitMapLength);
    for (int i=0;i<sketch.bitmaps.length;i++)
      for (int j=0;j<sketch.bitmaps[i].length;j++)
        this.bitmaps[i][j]=sketch.bitmaps[i][j];
  }

  public synchronized void insert(long keyIn)
  {
    Random tempRandom = new Random(keyIn);

    int bitmapChoice = tempRandom.nextInt(bitmaps.length);

    int x = 0;
    while(tempRandom.nextBoolean())
      ++x;
    if (x<this.bitMapLength)
      bitmaps[bitmapChoice][x] = true;
    else
      bitmaps[bitmapChoice][this.bitMapLength-1] = true;
   }

  public static void main(String[] args)
  {
    if((args == null) || (args.length != 1))
      {
        usage();
      }

    int m = -1;
    try
      {
        m = new Integer(args[0]).intValue();
      }
    catch(NumberFormatException e)
      {
        usage();
      }

    if(m <= 0)
      {
        usage();
      }

    int iterations = 100;
    double arithmetic_total = 0.0;
    double geometric_total = 0.0;

    double min = Double.POSITIVE_INFINITY;
    double max = Double.NEGATIVE_INFINITY;

    for(int i = 0; i < iterations; ++i)
      {

        CountSketch sketch = new CountSketch(m);
        Random random = new Random(i);

        for(int j = 0; j < 5000; ++j)
          {
            sketch.insert(random.nextInt());

            //System.out.println("inserting "+(j+1));
            //sketch.printSketch();
            //System.out.print("\n");
            sketch.SetKey();
            //System.out.print(sketch.key1+" "+sketch.key0+"\n");
          }
        double estimate = sketch.estimate();
        arithmetic_total += estimate;
        geometric_total += Math.log(estimate);

        if(estimate < min)
          min = estimate;

        if(estimate > max)
          max = estimate;
        System.out.print(estimate);
        System.out.print("\n");
        //System.out.println("estimate = " + estimate);
      }

    System.out.println("min = " + min);
    System.out.println("max = " + max);

    System.out.println("arithmetic_mean = " + (arithmetic_total / iterations));
    System.out.println("geometric_mean = " + Math.exp(geometric_total / iterations));
  }

  static void usage()
  {
    System.err.println("java CountSketch <m>");
    System.exit(1);
  }

}
