package SketchAggSensor.sketch;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */


public class Sketch
  {
    final static double varphi = 0.77351;
    public int numBitMap;
    public int bitMapLength = 16;
    protected boolean bitmaps[][];
    public int key0 =0;
    public int key1 =0;
    public int keyarr1[];
    public int keyarr0[];



    public Sketch()
    {
      this(1);
    }

    public Sketch(int mIn)
    {
      if(mIn < 1)
        {
          throw new IllegalArgumentException("m must be a positive integer");
        }
      this.numBitMap = mIn;
      bitmaps = new boolean[mIn][bitMapLength];
      keyarr1 = new int[mIn];
      keyarr0 = new int[mIn];
      for (int i=0;i<mIn;i++)
      {  for (int j=0;j<bitMapLength;j++)
        {  bitmaps[i][j]=false;
        }
        keyarr1[i] =0;
        keyarr0[i] =0;
      }
    }

    public Sketch (int mIn, int length)
    {
      this.numBitMap = mIn;
      this.bitMapLength = length;
      keyarr1 = new int[mIn];
      keyarr0 = new int[mIn];

      bitmaps = new boolean[mIn][bitMapLength];
      for (int i=0;i<mIn;i++)
      {  for (int j=0;j<bitMapLength;j++)
       {  bitmaps[i][j]=false;
       }
       keyarr1[i] =0;
       keyarr0[i] =0;
      }
    }

    // based on Flajolet and Martin pseudo-code
    public synchronized double estimate()
    {
      int total = 0;

      for(int i = 0; i < bitmaps.length; ++i)
        {
          int pos = 0;
          while(bitmaps[i][pos] == true)
          {  ++pos;
             if (pos>=this.bitMapLength)
               break;
          }
          total += pos;
        }

      double nmaps = (double) bitmaps.length;

      return (nmaps / varphi) * Math.pow(2.0, ((double) total) / nmaps);
    }

    public static Sketch cloneSketch( Sketch sketA)
    {
      Sketch outputSketch = new Sketch(sketA.numBitMap,sketA.bitMapLength);
      for (int i=0; i < sketA.bitmaps.length;i++)
        for (int j=0;j < sketA.bitmaps[i].length;j++)
          outputSketch.bitmaps[i][j]=sketA.bitmaps[i][j];
      return outputSketch;
    }

    public static Sketch mergeSketch (Sketch sketA, Sketch sketB)
    {
      Sketch outputSketch = new Sketch(sketA.numBitMap,sketA.bitMapLength);
      for (int i=0; i < sketA.bitmaps.length;i++)
      {  for (int j=0;j < sketA.bitmaps[i].length;j++)
         { if (sketA.bitmaps[i][j] == true || sketB.bitmaps[i][j]== true)
           {
                 outputSketch.bitmaps[i][j] = true;
           }
         }
      }
      return outputSketch;
    }

    public void printSketch ()
    {
      for (int i=0; i < this.bitmaps.length;i++)
      {  for (int j=0;j < this.bitmaps[i].length;j++)
         {    if (this.bitmaps[i][j])
                 System.out.print("1 ");
              else
                 System.out.print("0 ");
         }
         System.out.print("\n");
      }
    }

    public void SetKey()
    {

      for (int i=0; i < this.numBitMap;i++)
      { int j=0;
        int k= this.bitmaps[i].length-1;
        while (this.bitmaps[i][j]==true)
        {  j++;
           if (j == this.bitmaps[i].length)
             break;
        }
       keyarr1[i] = j;
       while (bitmaps[i][k]==false)
       {    k--;
             if (k<0)
                break;
       }
       keyarr0[i] = this.bitMapLength-1-k;
      }
      key1 = keyarr1[0];
      key0 = keyarr0[0];
      for (int i=0; i<this.numBitMap; i++)
      {
        if ( key1 >= keyarr1[i])
          key1 = keyarr1[i];
        if (key0 >= keyarr0[i])
          key0 = keyarr0[i];
      }
    }

    public void SetKey(int start, int stop)
    {
      key1=0;
      key0=0;
      int length = stop - start;
      int tmpkeyarr1[];
      int tmpkeyarr0[];
      tmpkeyarr1 = new int[length];
      tmpkeyarr0 = new int[length];
      for (int i = 0; i<length;i++)
      {
        tmpkeyarr1[i]=0;
        tmpkeyarr0[i]=0;
      }

      for (int i=start; i < stop;i++)
      { int j=0;
        int k= this.bitmaps[i].length-1;
        while (this.bitmaps[i][j]==true)
        {  j++;
           if (j == this.bitmaps[i].length)
             break;
        }
        keyarr1[i-start] = j;
        while (bitmaps[i][k]==false)
        {    k--;
             if (k<0)
               break;
        }
        keyarr0[i-start] = this.bitMapLength-1-k;
      }
      key1 = keyarr1[0];
      key0 = keyarr0[0];
      for (int i=0; i<length; i++)
      {
        if ( key1 >= keyarr1[i])
          key1 = keyarr1[i];
        if (key0 >= keyarr0[i])
          key0 = keyarr0[i];
      }
    }

    public void compressionANF(int start, int stop)
    {
      key1=0;
      key0=0;
      boolean tmpkeyarr[];
      int range = stop -start;
      int length = this.bitMapLength*range;
      tmpkeyarr = new boolean[length];
      int k=0;
      for (int j=0;j<this.bitMapLength;j++)
      {  for (int i=start;i<stop;i++)
         {
            tmpkeyarr[k++] = this.bitmaps[i][j];
         }
      }
      k=0;
      while (tmpkeyarr[k]==true)
      {  k++;
         if (k == length)
           break;
      }
      key1 = k;
      k = length-1;
      while (tmpkeyarr[k]==false)
      {    k--;
           if (k<0)
             break;
      }
      key0 = length-1-k;
    }
}
