package SketchAggSensor.sketch;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import java.util.Random;

public class SumSketch extends Sketch
{
  public SumSketch()
  {
    super();
  }

  public SumSketch(int mIn)
  {
    super(mIn);
  }

  public SumSketch(int mIn, int length)
  {
    super (mIn,length);
  }

  public SumSketch (Sketch sketch)
  {
    super(sketch.numBitMap,sketch.bitMapLength);
    for (int i=0;i<sketch.bitmaps.length;i++)
      for (int j=0;j<sketch.bitmaps[i].length;j++)
        this.bitmaps[i][j]=sketch.bitmaps[i][j];
  }

  static int pick_fill_length(int valueIn)
  {
    if(valueIn <= 10)
      return 0;
    else
      {
        double value = (double) valueIn;
        double log_value = Math.log(value);

        return (int) Math.floor(Math.log(value / (log_value * log_value)) / Math.log(2.0));
      }
  }

  // translation of kachitvichyanukul88binomial BINV code
 public int binomial_pick(int nIn, double pIn, Random randomIn)
  {
    double q = 1.0 - pIn;
    double s = pIn / q;
    double a = ((double) (nIn + 1)) * s;

    double r = Math.pow(q, (double) nIn);

    double u = randomIn.nextDouble();
    int x = 0;

    while(u > r)
    {
      u -= r;
      ++x;

      r *= (a / x) - s;
    }

    return x;
  }

  public synchronized void insert(long keyIn, int valueIn)
  {
    Random tempRandom = new Random(keyIn + ((long) valueIn));

    // add individual values so bulk adds are equally sized
    for(; valueIn % bitmaps.length != 0; --valueIn)
      {
        int bitmapChoice = tempRandom.nextInt(bitmaps.length);

        int x = 0;
        while(tempRandom.nextBoolean())
          ++x;
        if (x<this.bitMapLength)
          bitmaps[bitmapChoice][x] = true;
        else
          bitmaps[bitmapChoice][this.bitMapLength-1] = true;

      }

    int bulkAddSize = valueIn / bitmaps.length;
    int fill_length = pick_fill_length(bulkAddSize);

    for(int i = 0; i < bitmaps.length; ++i)
      {
        for(int j = 0; j < fill_length; ++j)
          bitmaps[i][j] = true;

        int remaining = ((fill_length == 0) ?
                         bulkAddSize :
                         binomial_pick(bulkAddSize, Math.pow(0.5, (double) fill_length), tempRandom));

        for(int j = 0; j < remaining; ++j)
          {
            int x = fill_length;
            while(tempRandom.nextBoolean())
              ++x;
            if (x<this.bitMapLength)
              bitmaps[i][x] = true;
            else
              bitmaps[i][this.bitMapLength-1] = true;
          }
      }
  }

  public static void main(String[] args)
  {
    if((args == null) || (args.length != 1))
      {
        usage();
      }

    int m = -1;
    try
      {
        m = new Integer(args[0]).intValue();
      }
    catch(NumberFormatException e)
      {
        usage();
      }

    if(m <= 0)
      {
        usage();
      }

    int iterations = 1;
    double arithmetic_total = 0.0;
    double geometric_total = 0.0;

    double min = Double.POSITIVE_INFINITY;
    double max = Double.NEGATIVE_INFINITY;

    for(int i = 0; i < iterations; ++i)
      {
        SumSketch sketch = new SumSketch(m);

        Random random = new Random(i);

        for(int j = 0; j < 900; ++j)
          {
            sketch.insert(random.nextInt(), 100);
            System.out.println("inserting "+(j+1));
            sketch.printSketch();
            System.out.print("\n");
          }

        double estimate = sketch.estimate();
        arithmetic_total += estimate;
        geometric_total += Math.log(estimate);

        if(estimate < min)
          min = estimate;

        if(estimate > max)
          max = estimate;

        System.out.println("estimate = " + estimate);
      }

    System.out.println("min = " + min);
    System.out.println("max = " + max);

    System.out.println("arithmetic_mean = " + (arithmetic_total / iterations));
    System.out.println("geometric_mean = " + Math.exp(geometric_total / iterations));
  }

  static void usage()
  {
    System.err.println("java SumSketch <m>");
    System.exit(1);
  }
}